<?php

namespace Drupal\entity_word\Controller;

use Drupal\Core\Controller\ControllerBase;
use PhpOffice\PhpWord\PhpWord;
use PhpOffice\PhpWord\Shared\Html;
use PhpOffice\PhpWord\Style\Font;
use PhpOffice\PhpWord\IOFactory;
use PhpOffice\PhpWord\Settings;

/**
 * Class EntityWordController.
 */
class EntityWordController extends ControllerBase {

  /**
   * Method for download node into word document.
   */
  public function nodeWord($node_id = NULL) {
    $node = \Drupal::entityTypeManager()->getStorage("node")->load($node_id);
    $language = \Drupal::languageManager()->getCurrentLanguage()->getId();
    // Get the data from config settings.
    $configFactory = \Drupal::service('config.factory');
    $config = $configFactory->get('entity_word.settings');
    $doc_filename = \Drupal::token()->replace($config->get('entity_word_filename'),
      [$node->getEntityTypeId() => $node], ['langcode' => $language]);

    // Instantiation of the class PhpWord.
    Settings::setOutputEscapingEnabled(TRUE);
    $phpWord = new PhpWord();
    $section = $phpWord->addSection(
      [
        'paperSize' => $config->get('entity_word_papersize'),
        'marginLeft'   => $config->get('entity_word_margin_left'),
        'marginRight'  => $config->get('entity_word_margin_right'),
        'marginTop'    => $config->get('entity_word_margin_top'),
        'marginBottom' => $config->get('entity_word_margin_bottom'),
        'headerHeight' => 0,
        'footerHeight' => 0
      ]
    );
    // Applying the font and style on title and body.
    $fontStyle = new Font();
    $fontStyle->setBold(TRUE);
    $fontStyle->setName($config->get('entity_word_font_family'));
    $fontStyle->setColor($config->get('entity_word_font_color'));
    $fontStyle->setSize($config->get('entity_word_font_size'));
    $textElement = $section->addText($node->title->value);
    $textElement->setFontStyle($fontStyle);
    $textElement = $section->addTextRun();
    $body_text = str_replace("<p>",
    "<p style='" . $config->get('entity_word_paragraph_style') . "'>",
    $node->body->value);
    Html::addHtml($section, $body_text, FALSE, FALSE);

    // Force download of the document.
    header('Content-Description: File Transfer');
    header('Content-Type: application/octet-stream');
    header('Content-Disposition: attachment;filename="' . $doc_filename . '"');
    header('Content-Transfer-Encoding: binary');
    header('Connection: Keep-Alive');
    header('Expires: 0');
    header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
    header('Pragma: public');
    $objWriter = IOFactory::createWriter($phpWord, 'Word2007');
    ob_clean();
    $objWriter->save('php://output');
    exit;
  }

}
